/* ----------------------------------------------------------------------- *
 *
 *   Copyright 2008 rPath, Inc. - All Rights Reserved
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 *   Boston MA 02110-1301, USA; either version 2 of the License, or
 *   (at your option) any later version; incorporated herein by reference.
 *
 * ----------------------------------------------------------------------- */

/*
 * le.h
 *
 * Write unaligned littleendian data
 */

#ifndef LE_H
#define LE_H

#include <inttypes.h>

#if defined(__i386__) || defined(__x86_64__)

/* Littleendian architecture, unaligned accesses permitted */

static inline uint16_t rdle16(const uint16_t *p)
{
	return *p;
}

static inline uint32_t rdle32(const uint32_t *p)
{
	return *p;
}

static inline void wrle16(uint16_t v, uint16_t *p)
{
	*p = v;
}

static inline void wrle32(uint32_t v, uint32_t *p)
{
	*p = v;
}

#else

static inline uint16_t rdle16(const uint16_t *p)
{
	const uint8_t *_p = (const uint8_t *)p;
	return _p[0] + (_p[1] << 8);
}

static inline uint32_t rdle32(const uint32_t *p)
{
	const uint8_t *_p = (const uint8_t *)p;
	return _p[0] + (_p[1] << 8) + (_p[2] << 16) + (_p[3] << 24);
}

static inline void wrle16(uint16_t v, uint16_t *p)
{
	uint8_t *_p = (uint8_t *)p;

	_p[0] = v;
	_p[1] = v >> 8;
}

static inline void wrle32(uint32_t v, uint32_t *p)
{
	uint8_t *_p = (uint8_t *)p;

	_p[0] = v;
	_p[1] = v >> 8;
	_p[2] = v >> 16;
	_p[3] = v >> 24;
}

#endif

#endif /* LE_H */
